import attrs from './attrs'
import klass from './class'
import events from './events'
import style from './style'
import transition from './transition'

export default [
  attrs,
  klass,
  events,
  style,
  transition
]
